"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var template_1 = __importDefault(require("@babel/template"));
var t = __importStar(require("@babel/types"));
var traverse_1 = __importDefault(require("@babel/traverse"));
var utils_1 = require("@compiled/utils");
var css_1 = require("@compiled/css");
var is_prop_valid_1 = __importDefault(require("@emotion/is-prop-valid"));
var buildCssVariablesProp = function (variables, transform) {
    if (transform === void 0) { transform = function (expression) { return expression; }; }
    return utils_1.unique(variables, function (item) { return item.name; }).map(function (variable) {
        return t.objectProperty(t.stringLiteral(variable.name), transform(variable.expression));
    });
};
var styledStyleProp = function (variables, transform) {
    var props = [t.spreadElement(t.identifier('style'))];
    return t.objectExpression(props.concat(buildCssVariablesProp(variables, transform)));
};
var styledTemplate = function (opts) {
    var nonceAttribute = opts.nonce ? "nonce={" + opts.nonce + "}" : '';
    var propsToDestructure = [];
    var styleProp = opts.variables.length
        ? styledStyleProp(opts.variables, function (node) {
            if (t.isArrowFunctionExpression(node)) {
                traverse_1.default(node, {
                    MemberExpression: function (path) {
                        if (t.isIdentifier(path.node.object) && path.node.object.name === 'props') {
                            var propertyAccessName = path.node.property;
                            if (is_prop_valid_1.default(propertyAccessName.name)) {
                                return;
                            }
                            if (!propsToDestructure.includes(propertyAccessName.name)) {
                                propsToDestructure.push(propertyAccessName.name);
                            }
                            path.replaceWith(propertyAccessName);
                        }
                    },
                }, opts.scope, undefined, opts.parentPath);
                return node.body;
            }
            return node;
        })
        : t.identifier('style');
    return template_1.default("\n  React.forwardRef(({\n    as: C = \"" + opts.tag + "\",\n    style,\n    " + propsToDestructure.map(function (prop) { return prop + ','; }).join('') + "\n    ...props\n  }, ref) => (\n    <CC>\n      <CS " + nonceAttribute + " hash=\"" + opts.hash + "\">{%%cssNode%%}</CS>\n      <C {...props} style={%%styleProp%%} ref={ref} className={\"" + opts.className + "\" + (props.className ? \" \" + props.className : \"\")} />\n    </CC>\n  ));\n", {
        plugins: ['jsx'],
    })({
        styleProp: styleProp,
        cssNode: t.arrayExpression(opts.css.map(function (style) { return t.stringLiteral(style); })),
    });
};
var compiledTemplate = function (opts) {
    var nonceAttribute = opts.nonce ? "nonce={" + opts.nonce + "}" : '';
    return template_1.default("\n  <CC>\n    <CS " + nonceAttribute + " hash=\"" + opts.hash + "\">{%%cssNode%%}</CS>\n    {%%jsxNode%%}\n  </CC>\n  ", {
        plugins: ['jsx'],
    })({
        jsxNode: opts.jsxNode,
        cssNode: t.arrayExpression(opts.css.map(function (style) { return t.stringLiteral(style); })),
    });
};
exports.joinExpressions = function (left, right, spacer) {
    if (spacer === void 0) { spacer = t.stringLiteral(' '); }
    return t.binaryExpression('+', left, spacer ? t.binaryExpression('+', spacer, right) : right);
};
exports.conditionallyJoinExpressions = function (left, right) {
    return t.binaryExpression('+', left, t.conditionalExpression(right, t.binaryExpression('+', t.stringLiteral(' '), right), t.stringLiteral('')));
};
exports.buildStyledComponent = function (opts) {
    var cssHash = utils_1.hash(opts.cssOutput.css);
    var className = "cc-" + cssHash;
    var cssRules = css_1.transformCss("." + className, opts.cssOutput.css);
    return styledTemplate({
        className: className,
        hash: cssHash,
        tag: opts.tagName,
        css: cssRules,
        variables: opts.cssOutput.variables,
        parentPath: opts.parentPath,
        scope: opts.scope,
    });
};
exports.importSpecifier = function (name, localName) {
    return t.importSpecifier(t.identifier(name), t.identifier(localName || name));
};
exports.buildCompiledComponent = function (opts) {
    var cssHash = utils_1.hash(opts.cssOutput.css);
    var className = "cc-" + cssHash;
    var cssRules = css_1.transformCss("." + className, opts.cssOutput.css);
    var classNameProp = opts.node.openingElement.attributes.find(function (prop) {
        return t.isJSXAttribute(prop) && prop.name.name === 'className';
    });
    if (classNameProp && classNameProp.value) {
        var classNameExpression = t.isJSXExpressionContainer(classNameProp.value)
            ? classNameProp.value.expression
            : classNameProp.value;
        var newClassNameValue = t.isStringLiteral(classNameExpression)
            ? exports.joinExpressions(t.stringLiteral(className), classNameExpression)
            : exports.conditionallyJoinExpressions(t.stringLiteral(className), classNameExpression);
        classNameProp.value = t.jsxExpressionContainer(newClassNameValue);
    }
    else {
        opts.node.openingElement.attributes.push(t.jsxAttribute(t.jsxIdentifier('className'), t.stringLiteral(className)));
    }
    if (opts.cssOutput.variables.length) {
        var stylePropIndex_1 = -1;
        var styleProp = opts.node.openingElement.attributes.find(function (prop, index) {
            if (t.isJSXAttribute(prop) && prop.name.name === 'style') {
                stylePropIndex_1 = index;
                return true;
            }
            return false;
        });
        var dynamicStyleProperties_1 = utils_1.unique(opts.cssOutput.variables, function (item) { return item.name; }).map(function (variable) {
            return t.objectProperty(t.stringLiteral(variable.name), variable.expression);
        });
        if (styleProp) {
            // Remove the pre-existing style prop - we're going to redefine it soon.
            opts.node.openingElement.attributes.splice(stylePropIndex_1, 1);
            if (styleProp.value &&
                t.isJSXExpressionContainer(styleProp.value) &&
                !t.isJSXEmptyExpression(styleProp.value.expression)) {
                // If it's not an object we just spread the expression into the object
                if (!t.isObjectExpression(styleProp.value.expression)) {
                    dynamicStyleProperties_1.splice(0, 0, t.spreadElement(styleProp.value.expression));
                }
                else {
                    // Else it's an object! So we want to place each property into the object
                    styleProp.value.expression.properties.forEach(function (prop, index) {
                        if (t.isObjectMethod(prop)) {
                            return;
                        }
                        // ... in the order they were defined! (So we're using index here to do just that).
                        dynamicStyleProperties_1.splice(index, 0, prop);
                    });
                }
            }
        }
        opts.node.openingElement.attributes.push(t.jsxAttribute(t.jsxIdentifier('style'), t.jsxExpressionContainer(t.objectExpression(dynamicStyleProperties_1))));
    }
    return compiledTemplate({
        jsxNode: opts.node,
        hash: cssHash,
        css: cssRules,
        nonce: opts.nonce,
    });
};
//# sourceMappingURL=ast-builders.js.map